/* application.cpp */

/* Copyright (C) 2011-2015  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "application.h"

#include <QMetaType>
#include <QSharedMemory>


Application::Application(int & argc, char ** argv) :
	QApplication(argc, argv),
	SayonaraClass()
{
	QFont font = this->font();
	font.setPointSize(8);
	this->setFont(font);
}

void Application::check_for_crash(){

	QString error_file = Helper::getErrorFile();

	if(!QFile::exists(error_file)) return;

	QString info_text;
	QString mail;

	if(!Helper::read_file_into_str(error_file, mail)){
		mail = "";
		mail.prepend("mailto:luciocarreras@gmail.com?subject=Sayonara Crash&amp;body=Hi Lucio,\n\nhere is the trace for a Sayonara crash\n\n");
	}

	else{
		mail.prepend("mailto:luciocarreras@gmail.com?subject=Sayonara Crash&amp;body=Hi Lucio,\n\nhere is the trace for a Sayonara crash\n\n");
		mail.append("\n\nI hope this will not happen again...");
	}

	info_text = QString("Sayonara seems to have crashed the last time<br />") +
			"Please send " +
			Helper::createLink(error_file, error_file) +
			" in " + Helper::createLink(Helper::getSayonaraPath(), Helper::getSayonaraPath()) +
			" to " + Helper::createLink("luciocarreras@gmail.com", mail);

	GlobalMessage::getInstance()->info(info_text);

	QFile f(error_file);
	f.open(QIODevice::ReadOnly);
	if(!f.isOpen()){
		sp_log(Log::Warning) << "Cannot oopen " << error_file;
		return;
	}

	f.remove();
	f.close();

	return;
}

bool Application::init(QTranslator* translator, const QStringList& files_to_play) {

	bool success = this->installTranslator(translator);
	if(!success){
		sp_log(Log::Warning) << "Cannot install translator";
	}

	check_for_crash();

	QString version = QString(SAYONARA_VERSION);
	_settings->set(Set::Player_Version, version);

	player              = new GUI_Player(translator);
	_dbus				= new DBusHandler(this);
	stream_server       = new StreamServer(this);
	ui_playlist         = new GUI_Playlist(player);

	ui_level            = new GUI_LevelPainter(tr("Le&vel"), nullptr);
	ui_spectrum         = new GUI_Spectrum(tr("&Spectrum"), nullptr);
	ui_stream           = new GUI_Stream(tr("&Webstreams"), nullptr);
	ui_podcasts         = new GUI_Podcasts(tr("P&odcasts"), nullptr);
	ui_eq               = new GUI_Equalizer(tr("&Equalizer"), nullptr);
	ui_playlist_chooser = new GUI_PlaylistChooser(tr("Pla&ylists"), nullptr);
	ui_audioconverter   = new GUI_AudioConverter(tr("&mp3 Converter"), nullptr);
	ui_bookmarks        = new GUI_Bookmarks(tr("&Bookmarks"), nullptr);
	ui_speed			= new GUI_Speed(tr("Spee&d"), nullptr);
	ui_broadcast		= new GUI_Broadcast(tr("&Broadcast"), stream_server, nullptr);

	_pph = new PlayerPluginHandler(this);

	_pph->addPlugin(ui_level);
	_pph->addPlugin(ui_spectrum);
	_pph->addPlugin(ui_eq);
	_pph->addPlugin(ui_stream);
	_pph->addPlugin(ui_podcasts);
	_pph->addPlugin(ui_playlist_chooser);
	_pph->addPlugin(ui_audioconverter);
	_pph->addPlugin(ui_bookmarks);
	_pph->addPlugin(ui_speed);
	_pph->addPlugin(ui_broadcast);

	sp_log(Log::Info) << "Plugin " << ui_level->getVisName();
	sp_log(Log::Info) << "Plugin " << ui_stream->getVisName();
	sp_log(Log::Info) << "Plugin " << ui_eq->getVisName();
	sp_log(Log::Info) << "Plugin " << ui_playlist_chooser->getVisName();
	sp_log(Log::Info) << "Plugin " << ui_podcasts->getVisName();
	sp_log(Log::Info) << "Plugin " << ui_audioconverter->getVisName();
	sp_log(Log::Info) << "Plugin " << ui_bookmarks->getVisName();
	sp_log(Log::Info) << "Plugin " << ui_speed->getVisName();
	sp_log(Log::Info) << "Plugin " << ui_broadcast->getVisName();

	PlayManager::getInstance()->pause();

	LibraryPluginLoader* library_plugin_loader = new LibraryPluginLoader(this);

	player->setWindowTitle(QString("Sayonara ") + version);
	player->setWindowIcon(Helper::getIcon("logo.png"));
	player->set_playlist(ui_playlist);
	player->set_libraries(library_plugin_loader, 0);
	player->set_player_plugin_handler(_pph);
	player->show();
	player->ui_loaded();

	if(_settings->get(Set::PL_StartPlaying)){
		PlayManager::getInstance()->play();
	}

	else{
		PlayManager::getInstance()->pause();
	}

	if(files_to_play.size() > 0) {
		PlaylistHandler::getInstance()->create_playlist(files_to_play, tr("Library"));
	}

	return true;
}

Application::~Application() {
	CDatabaseConnector::getInstance()->store_settings();
	CDatabaseConnector::getInstance()->close_db();
}

