/* Bookmarks.cpp */

/* Copyright (C) 2011-2014  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#include "Bookmarks.h"
#include <algorithm> // std::max

Bookmarks::Bookmarks(QObject *parent) :
	QObject(parent)
{

	_play_manager = PlayManager::getInstance();
	_db = CDatabaseConnector::getInstance();
	_bookmarks.clear();
	_cur_time = 0;
	_prev_idx = -1;
	_next_idx = -1;

	connect(_play_manager, SIGNAL(sig_track_changed(const MetaData&)),
			this, SLOT(track_changed(const MetaData&)));

	connect(_play_manager, SIGNAL(sig_position_changed_ms(quint64)),
			this, SLOT(pos_changed_ms(quint64)));

	connect(_play_manager, SIGNAL(sig_playstate_changed(PlayManager::PlayState)),
			this, SLOT(playstate_changed(PlayManager::PlayState)));
}


bool Bookmarks::is_md_extern()
{
	return (_md.id < 0);
}

void Bookmarks::sort_bookmarks(){
	auto lambda = [](const Bookmark& bm1, const Bookmark& bm2){
		return bm1.get_time() < bm2.get_time();
	};

	std::sort(_bookmarks.begin(), _bookmarks.end(), lambda);
}

void Bookmarks::reload_bookmarks(){
	_bookmarks.clear();

	QMap<quint32, QString> bookmarks;
	_db->searchBookmarks(_md.id, bookmarks);

	for(const quint32& key : bookmarks.keys()){
		_bookmarks << Bookmark(key, bookmarks[key]);
	}

	sort_bookmarks();

	emit sig_bookmarks_changed(_bookmarks);
}


bool Bookmarks::save()
{
	if(_cur_time == 0) return false;
	QString name = Helper::cvt_ms_to_string(_cur_time * 1000, true, true, false);
	bool success = _db->insertBookmark(_md.id, _cur_time, name);

	if(success){
		reload_bookmarks();
	}

	return success;
}


bool Bookmarks::remove(int idx){
	bool success = _db->removeBookmark(_md.id, _bookmarks[idx].get_time());

	if(success){
		reload_bookmarks();
	}

	return success;
}


bool Bookmarks::remove_all(){
	bool success = _db->removeAllBookmarks(_md.id);

	if(success){
		reload_bookmarks();
	}

	return success;
}


bool Bookmarks::jump_to(int idx){
	_play_manager->seek_abs_ms(_bookmarks[idx].get_time() * 1000);
	return true;
}

bool Bookmarks::jump_next(){
	if(_next_idx < 0 || _next_idx > _bookmarks.size() - 1){
		emit sig_next_changed(Bookmark(), false);
		return false;
	}

	quint32 new_time = _bookmarks[_next_idx].get_time();
	_play_manager->seek_abs_ms(new_time * 1000);

	return true;
}

bool Bookmarks::jump_prev(){
	if(_prev_idx < 0 || _prev_idx >= _bookmarks.size()){
		emit sig_prev_changed(Bookmark(), false);
		return false;
	}

	quint32 new_time = _bookmarks[_prev_idx].get_time();
	_play_manager->seek_abs_ms(new_time * 1000);

	return true;
}


void Bookmarks::pos_changed_ms(quint64 pos_ms){

	_cur_time = (quint32) (pos_ms / 1000);
	if(_cur_time >= _loop_end && _loop_end != 0){
		jump_prev();
		return;
	}

	if(_bookmarks.size() == 0){
		return;
	}

	_prev_idx=-1;
	_next_idx=-1;

	for(int i=0; i<_bookmarks.size(); i++){

		quint32 time = _bookmarks[i].get_time();


		if(time < _cur_time){
			_prev_idx = i;
		}

		if(time > _cur_time){
			if(_next_idx == -1){
				_next_idx = i;
				break;
			}
		}
	}

	if(_prev_idx >= 0 && _prev_idx < _bookmarks.size()){
		emit sig_prev_changed(_bookmarks[_prev_idx], true);
	}
	else{
		emit sig_prev_changed(Bookmark(), false);
	}

	if(_next_idx >= 0 && _next_idx < _bookmarks.size()){
		emit sig_next_changed(_bookmarks[_next_idx], true);
	}
	else{
		emit sig_next_changed(Bookmark(), false);
	}
}


void Bookmarks::track_changed(const MetaData& md)
{
	_md = md;
	if(md.id < 0){
		return;
	}

	_bookmarks.clear();
	QMap<quint32, QString> bookmarks;
	_db->searchBookmarks(md.id, bookmarks);

	for(const quint32& key : bookmarks.keys()){
		_bookmarks << Bookmark(key, bookmarks[key]);
	}

	sort_bookmarks();

	emit sig_bookmarks_changed(_bookmarks);
	emit sig_prev_changed(Bookmark(), false);
	emit sig_next_changed(Bookmark(), false);
}


void Bookmarks::playstate_changed(PlayManager::PlayState state){
	Q_UNUSED(state)
}

bool Bookmarks::set_loop(bool b){
	if(b){
		if(_prev_idx >= 0 && _prev_idx < _bookmarks.size()){
			_loop_start = _bookmarks[_prev_idx].get_time();
		}

		else {
			_loop_start = 0;
			_loop_end = 0;
			return false;
		}

		if(_next_idx >= 0 && _next_idx < _bookmarks.size()){
			_loop_end = _bookmarks[_next_idx].get_time();
		}

		else{
			_loop_start = 0;
			_loop_end = 0;
			return false;
		}
	}

	else{
		_loop_start = 0;
		_loop_end = 0;
		return false;
	}

	return true;
}

QList<Bookmark> Bookmarks::get_all_bookmarks(){
	return _bookmarks;
}

