/* GSTEngineCallbacks.cpp */

/* Copyright (C) 2011-2015  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "GSTPlaybackEngine.h"
#include "GSTEngineHelper.h"

#include "Helper/globals.h"

#include <algorithm>		// std::min
#include <gst/gst.h>
#include <gst/gsturi.h>
#include <gst/app/gstappsink.h>
#include <gst/gstbuffer.h>
#include <gst/gstelement.h>


double arr[2];

gboolean
EngineCallbacks::level_handler (GstBus * bus, GstMessage * message, gpointer data) {

	Q_UNUSED(bus);

	Engine* engine = (Engine*) data;

	if(!engine) {
		return true;
	}

	GValueArray*			rms_arr;
	const GstStructure*		structure;
	const gchar*			name;
	const GValue*			peak_value;
	guint					n_peak_elements;

	structure = gst_message_get_structure (message);
	if(!structure) {
		sp_log(Log::Warning) << "structure is null";
		return true;
	}

	name = gst_structure_get_name(structure);
    if ( strcmp(name, "level") != 0 ) {
		return true;
    }

	peak_value = gst_structure_get_value(structure, "peak");
	if(!peak_value) {
		return true;
	}

	rms_arr = (GValueArray*) g_value_get_boxed(peak_value);

	n_peak_elements = rms_arr->n_values;
	if(n_peak_elements == 0) {
		return true;
    }

	n_peak_elements = std::min((guint) 2, n_peak_elements);
	for(guint i=0; i<n_peak_elements; i++) {

		double d;
		const GValue* val;

		val = rms_arr->values + i;

		if(!G_VALUE_HOLDS_DOUBLE(val)) {
			sp_log(Log::Debug) << "Could not find a double";
			break;
		}

		d = g_value_get_double(val);
		if(d < 0){
			arr[i] = d;
		}
	}

	if(n_peak_elements >= 2) {
		engine->set_level(arr[0], arr[1]);
	}

	else if(n_peak_elements == 1) {
		engine->set_level(arr[0], arr[0]);
	}

	return true;
}


gboolean
EngineCallbacks::spectrum_handler (GstBus * bus, GstMessage * message, gpointer data) {

	Q_UNUSED(bus);
    Q_UNUSED(data);

	Engine* engine = (Engine*) data;
	if(!engine) {
		return true;
	}

	const GstStructure*		structure;
	const gchar*			structure_name;
	const GValue*			magnitudes;
	QList<float>			spectrum_vals;


	structure = gst_message_get_structure(message);
	if(!structure) {
		return true;
    }

	structure_name = gst_structure_get_name(structure);
	if( strcmp(structure_name, "spectrum") != 0 ) {
		return true;
    }

	magnitudes = gst_structure_get_value (structure, "magnitude");
	for (guint i = 0; i < N_BINS; ++i) {

        float f;
		const GValue* mag;

		mag = gst_value_list_get_value(magnitudes, i);
		if(!mag) {
			continue;
		}

		f = (g_value_get_float(mag) + crop_spectrum_at) / (crop_spectrum_at * 1.0f);
		spectrum_vals << f;
    }

	engine->set_spectrum(spectrum_vals);

	return true;
}


int blupp=0;
gboolean EngineCallbacks::bus_state_changed(GstBus *bus, GstMessage *msg, gpointer data) {

    (void) bus;

	Engine* engine = (Engine*) data;

	int msg_type = GST_MESSAGE_TYPE(msg);
	quint32 br;
	MetaData md;

	switch (msg_type) {

        case GST_MESSAGE_EOS:
			if (!engine) {
				break;
			}

            engine->set_track_finished();

            break;

        case GST_MESSAGE_ELEMENT:

			if(!engine) {
				break;
			}

			if(engine->get_show_spectrum()){
				return spectrum_handler(bus, msg, engine);
			}

			if(engine->get_show_level()){
				return level_handler(bus, msg, engine);
			}

            break;

		case GST_MESSAGE_TAG:

			GstTagList* tags;
			gchar* title;
			bool success;

            tags = nullptr;
            gst_message_parse_tag(msg, &tags);

			if(!tags){
				break;
			}

			success = gst_tag_list_get_uint(tags, GST_TAG_BITRATE, &br);
			if(success){
				engine->update_bitrate((br / 1000) * 1000);
			}

			success = gst_tag_list_get_string(tags, GST_TAG_TITLE, (gchar**) &title);
			if(success){
				md.title = title;
				g_free(title);
				engine->update_md(md);
			}

			gst_tag_list_unref(tags);

			break;

		case GST_MESSAGE_STATE_CHANGED:
			GstState old_state, new_state, pending_state;

			gst_message_parse_state_changed(msg, &old_state, &new_state, &pending_state);

			if( new_state == GST_STATE_PLAYING){
				engine->set_track_ready();
			}

			break;

		case GST_MESSAGE_BUFFERING:
			gint percent;
			gst_message_parse_buffering(msg, &percent);
			engine->buffering(percent);
			break;

		case GST_MESSAGE_DURATION_CHANGED:
			engine->update_duration();
			break;

        case GST_MESSAGE_ERROR:
            GError *err;

            gst_message_parse_error(msg, &err, nullptr);

			sp_log(Log::Error) << "Engine " << engine->get_name() << ": GST_MESSAGE_ERROR: " << err->message << ": "
                     << GST_MESSAGE_SRC_NAME(msg);

			if(engine){
				engine->set_track_finished();
			}

            g_error_free(err);

            break;

        default:

            if(engine) {
                engine->unmute();
            }

            break;
    }

    return true;
}
