/* GSTPlaybackEngine.cpp */

/* Copyright (C) 2011-2015  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "Helper/Tagging/id3.h"
#include "GSTPlaybackEngine.h"

#include <QUrl>

#define WATCH_INTERVAL 200
bool _debug = false;


/*****************************************************************************************/
/* Engine */
/*****************************************************************************************/

GSTPlaybackEngine::GSTPlaybackEngine(QObject* parent) :
	Engine(parent)
{

	ENGINE_DEBUG;

	_pipeline = nullptr;
	_other_pipeline = nullptr;

	_md.id = -1;

	_caps = new MyCaps();
	_caps->set_parsed(false);

	_name = PLAYBACK_ENGINE;

	_cur_pos_ms = 0;
	_playing_stream = false;
	_sr_wanna_record = false;

	_stream_recorder = new StreamRecorder(this);

	_gapless_state = GaplessState::NoGapless;

	REGISTER_LISTENER(Set::Engine_SR_Active, _change_sr_active);
}


GSTPlaybackEngine::~GSTPlaybackEngine() {

	delete _pipeline;
	delete _stream_recorder;
	delete _caps;
}


bool GSTPlaybackEngine::init() {

	gst_init(0, 0);

	_pipeline = new GSTPlaybackPipeline(this);
	if(!_pipeline->init()){
		return false;
	}

	_other_pipeline = nullptr;

	connect(_pipeline, SIGNAL(sig_about_to_finish(qint64)), this, SLOT(set_about_to_finish(qint64)));
	connect(_pipeline, SIGNAL(sig_pos_changed_ms(qint64)), this, SLOT(set_cur_position_ms(qint64)));
	connect(_pipeline, SIGNAL(sig_data(uchar*, quint64)), this, SLOT(new_data(uchar*, quint64)));

	REGISTER_LISTENER(Set::Engine_Gapless, _change_gapless);
	return true;
}


void GSTPlaybackEngine::change_track_gapless(const MetaData& md) {

	set_uri(md);
	_md_gapless = md;
}


void GSTPlaybackEngine::change_track(const QString& filepath) {

	MetaData md;
	md.set_filepath(filepath);
	if( !ID3::getMetaDataOfFile(md) ) {
		stop();
		return;
	}

	change_track(md);
}


void GSTPlaybackEngine::change_track(const MetaData& md) {

	bool success = false;
	emit sig_pos_changed_s(0);

	if( _gapless_state == GaplessState::AboutToFinish ) {
		change_track_gapless(md);
		_gapless_state = GaplessState::TrackFetched;
		return;
	}

	success = set_uri(md);
	if (!success){
		return;
	}

	_md = md;
	_caps->set_parsed(false);
	_cur_pos_ms = 0;
}



bool GSTPlaybackEngine::set_uri(const MetaData& md) {

	bool success = false;
	QUrl url;
	gchar* uri;

	if(_uri){
		g_free(_uri);
		_uri = nullptr;
	}

	_playing_stream = Helper::is_www(md.filepath());

	// stream, but don't wanna record
	// stream is already uri
	if (_playing_stream) {

		_pipeline->set_sr_path("");

		url = QUrl(md.filepath());
		uri = url.toString().toLocal8Bit().data();
	}

	// no stream (not quite right because of mms, rtsp or other streams
	// normal filepath -> no uri
	else if (!md.filepath().contains("://")) {

		url = QUrl::fromLocalFile(md.filepath());
		uri = url.toString().toUtf8().data();
	}

	else {
		uri = md.filepath().toUtf8().data();
	}

	_uri = g_strdup(uri);

	success = _pipeline->set_uri(_uri);

	return success;
}


void GSTPlaybackEngine::play() {

	if(_gapless_state == GaplessState::AboutToFinish ||
	   _gapless_state == GaplessState::TrackFetched)
	{
		return;
	}

	_pipeline->play();
	_stream_recorder->changeTrack(_md);

    if(_sr_wanna_record){
        QString filename = _stream_recorder->get_dst_file();
        _pipeline->set_sr_path(filename);
	}
}


void GSTPlaybackEngine::stop() {

	_gapless_state = GaplessState::NoGapless;

	_pipeline->stop();

	if(_other_pipeline){
		_other_pipeline->stop();
	}

	emit sig_pos_changed_s(0);
}


void GSTPlaybackEngine::pause() {
    _pipeline->pause();
}


void GSTPlaybackEngine::jump_abs_ms(quint64 pos_ms) {

	_pipeline->seek_abs(pos_ms * MIO);
}

void GSTPlaybackEngine::jump_rel_ms(quint64 ms) {

	quint64 new_time_ms = _pipeline->get_position_ms() + ms;
	_pipeline->seek_abs(new_time_ms * MIO);
}


void GSTPlaybackEngine::jump_rel(double percent) {

	_pipeline->seek_rel(percent, _md.length_ms * MIO);
}


void GSTPlaybackEngine::eq_changed(int band, int val) {

	double new_val = 0;
	new_val = val * 1.0;
	if (val > 0) {
		new_val = val * 0.25;
	}

    else{
        new_val = val * 0.75;
    }

	QString band_name = QString("band") + QString::number(band);
	_pipeline->set_eq_band(band_name, new_val);

	if(_other_pipeline){
		_other_pipeline->set_eq_band(band_name, new_val);
	}
}


MyCaps* GSTPlaybackEngine::get_caps() const{
	return _caps;
}

void GSTPlaybackEngine::buffering(int progress)
{
	emit sig_buffering(progress);
}


void GSTPlaybackEngine::set_cur_position_ms(qint64 pos_ms) {

	// the new track is requested so we won't display the position
	// of the old track, because probably the new track is already
	// displayed and active in playlist
	if((_gapless_state == GaplessState::AboutToFinish) ||
	   (_gapless_state == GaplessState::TrackFetched))
	{
		emit sig_pos_changed_s(0);
		return;
	}

	if(pos_ms < 0 && Helper::is_www(_md.filepath())){
		return;
	}

	qint32 pos_sec;
	qint32 cur_pos_sec;

	pos_sec = pos_ms / 1000;
	cur_pos_sec = _cur_pos_ms / 1000;

    if ( cur_pos_sec == pos_sec ){
        return;
    }

	_cur_pos_ms = pos_ms;

	emit sig_pos_changed_s( pos_sec );
}

void GSTPlaybackEngine::set_track_ready(){

	emit sig_track_ready();
}


void GSTPlaybackEngine::set_track_finished() {

	// fetch new track
	if( _gapless_state == GaplessState::NoGapless ) {
		emit sig_track_finished();
	}

	// track already fetched, stop the pipeline
	// new pipeline will be activated automatically
	// by timer
	else{
		_gapless_state = GaplessState::Playing;

		_other_pipeline->stop();

		_md = _md_gapless;
		_cur_pos_ms = 0;
		_caps->set_parsed(false);
	}

	emit sig_pos_changed_ms(0);
}


void GSTPlaybackEngine::set_about_to_finish(qint64 time2go) {

	if( _gapless_state == GaplessState::NoGapless ||
		_gapless_state == GaplessState::AboutToFinish )
	{
		return;
	}

	// switch pipelines
	std::swap(_pipeline, _other_pipeline);

	_gapless_state = GaplessState::AboutToFinish;
	_pipeline->start_timer(time2go);

	emit sig_track_finished();
	emit sig_pos_changed_ms(0);
}


void GSTPlaybackEngine::unmute() {
	_pipeline->unmute();

	if(_other_pipeline){
		_other_pipeline->unmute();
	}
}


void GSTPlaybackEngine::_change_gapless() {

	bool gapless = _settings->get(Set::Engine_Gapless);

	if(gapless) {

		if(!_other_pipeline) {

			_other_pipeline = new GSTPlaybackPipeline(this);
			if(!_other_pipeline->init()){
				_gapless_state = GaplessState::NoGapless;
				return;
			}

			connect(_other_pipeline, SIGNAL(sig_about_to_finish(qint64)), this, SLOT(set_about_to_finish(qint64)));
			connect(_other_pipeline, SIGNAL(sig_pos_changed_ms(qint64)), this, SLOT(set_cur_position_ms(qint64)));
			connect(_other_pipeline, SIGNAL(sig_data(uchar*, quint64)), this, SLOT(new_data(uchar*, quint64)));
		}

		_gapless_state = GaplessState::Playing;
	}

	else {
		_gapless_state = GaplessState::NoGapless;
	}
}


void GSTPlaybackEngine::psl_set_speed(float f) {
	_pipeline->set_speed(f);

    if(_other_pipeline){
        _other_pipeline->set_speed(f);
    }
}


void GSTPlaybackEngine::_change_sr_active(){
	_sr_active = _settings->get(Set::Engine_SR_Active);
    if(!_sr_active){
        record_button_toggled(false);
    }
}


void GSTPlaybackEngine::record_button_toggled(bool b){
    Engine::record_button_toggled(b);

    QString dst_file;

    _stream_recorder->activate(b);
    _stream_recorder->changeTrack(_md);

    dst_file = _stream_recorder->get_dst_file();
    if(!_pipeline) return;

    _pipeline->set_sr_path(dst_file);
}


void GSTPlaybackEngine::update_md(const MetaData& md){

	if(!Helper::is_www( _md.filepath() )) {
		return;
	}

	if(md.title.isEmpty()) {
		return;
	}

	if(md.title == _md.title) {
		return;
	}

	sp_log(Log::Debug) << "Track " << md.filepath() << " changed";

	_cur_pos_ms = 0;
	_md.title = md.title;

	emit sig_md_changed(_md);

	_stream_recorder->changeTrack(_md);
	_pipeline->set_sr_path(_stream_recorder->get_dst_file());
}


void GSTPlaybackEngine::update_duration() {

	_pipeline->refresh_duration();

	if( _gapless_state == GaplessState::AboutToFinish ||
		_gapless_state == GaplessState::TrackFetched )
	{
		return;
	}

	qint64 duration_ms = _pipeline->get_duration_ms();
	quint32 duration_s = duration_ms / 1000;
	quint32 md_duration_s = _md.length_ms / 1000;

	if(duration_s <= 0) return;
	if(duration_s == md_duration_s) return;
	if(duration_s > 15000) return;

	_md.length_ms = duration_ms;

	emit sig_dur_changed(_md);
}


void GSTPlaybackEngine::update_bitrate(quint32 br){

	if( br / 1000 <= 0) {
		return;
	}

	if( br / 1000 == _md.bitrate / 1000) {
		return;
	}

	_md.bitrate = br;
	emit sig_br_changed(_md);
}
