/* StreamWriter.cpp */

/* Copyright (C) 2011-2015  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#include "StreamWriter.h"
#include "StreamHttpParser.h"
#include "StreamDataSender.h"

#include "Helper/Helper.h"

static char padding[256];

StreamWriter::StreamWriter(QTcpSocket* socket, const MetaData& md) : SayonaraClass()
{

	_sender = new StreamDataSender(socket);
	_parser = new StreamHttpParser();

	reset();

	memset(padding, 0, 256);

	_stream_title = md.title + " by " + md.artist;
	_socket = socket;

	connect(socket, SIGNAL(disconnected()), this, SLOT(socket_disconnected()));
	connect(socket, SIGNAL(readyRead()), this, SLOT(data_available()));
	connect(socket, SIGNAL(stateChanged(QAbstractSocket::SocketState)),
			this, SLOT(socket_state_changed(QAbstractSocket::SocketState)));

}

StreamWriter::~StreamWriter(){

	delete _parser;
	delete _sender;
}


QString StreamWriter::get_ip(){
	return _socket->peerAddress().toString();
}


int StreamWriter::get_sd(){
	return _socket->socketDescriptor();
}


void StreamWriter::reset(){
	_sent_bytes = 0;
	_send_data = false;
	_dismissed = false;
	_stream_title = "";
	_user_agent = "";
}


HttpAnswer StreamWriter::parse_message(){

	HttpAnswer status;
	status = _parser->parse(_socket->readAll());

	_user_agent = _parser->get_user_agent();

	return status;
}




bool StreamWriter::send_data(const uchar* data, quint64 size){

	if(!_send_data) {
		return true;
	}

	if(_dismissed){
		return _sender->send_trash();
	}

	if(_parser->is_icy()){
		return _sender->send_icy_data(data, size, _stream_title);
	}

	else{
		return _sender->send_data(data, size);
	}
}

bool StreamWriter::send_playlist(){
	return _sender->send_playlist(_parser->get_host(), _socket->localPort());
}

bool StreamWriter::send_favicon(){
	return _sender->send_favicon();
}

bool StreamWriter::send_metadata(){

	return _sender->send_metadata(_stream_title);
}

bool StreamWriter::send_bg(){
	return _sender->send_bg();
}

bool StreamWriter::send_html5(){

	return _sender->send_html5(_stream_title);
}


bool StreamWriter::send_header(bool reject){
	return _sender->send_header(reject, _parser->is_icy());
}




void StreamWriter::change_track(const MetaData& md){

	_stream_title = md.title + " by " + md.artist;
	sp_log(Log::Debug) << "Update stream title " << _stream_title;
}



void StreamWriter::disable(){

    if(_dismissed) return;

	_dismissed = true;
}


void StreamWriter::enable(){

    _send_data = true;
}


void StreamWriter::disconnect(){
	
	disable();
	
	_socket->disconnectFromHost();
}


void StreamWriter::socket_disconnected(){

	reset();
	emit sig_disconnected(this);
}


void StreamWriter::data_available(){

	sp_log(Log::Debug) << "***New data available***";

	HttpAnswer answer = parse_message();
	QString ip = get_ip();
	bool success;
	bool close_connection = true;


	switch(answer){

		case HttpAnswer::Fail:
		case HttpAnswer::Reject:
			sp_log(Log::Debug) << "Rejected: " << _parser->get_user_agent() << ": " << get_ip();
			send_header(true);
			break;

		case HttpAnswer::Ignore:
			sp_log(Log::Debug) << "ignore...";
			break;

		case HttpAnswer::Playlist:
			sp_log(Log::Debug) << "Asked for playlist";
			send_playlist();
			break;

		case HttpAnswer::HTML5:
			sp_log(Log::Debug) << "Asked for html5";
			send_html5();

			break;

		case HttpAnswer::BG:
			sp_log(Log::Debug) << "Asked for background";
			send_bg();
			break;

		case HttpAnswer::Favicon:
			sp_log(Log::Debug) << "Asked for favicon";
			send_favicon();
			break;

		case HttpAnswer::MetaData:
			sp_log(Log::Debug) << "Asked for metadata";
			send_metadata();
			break;

		default:
			close_connection = false;
			sp_log(Log::Debug) << "Accepted: " << _parser->get_user_agent() << ": " << ip;
			success = send_header(false);
			if(success){
				enable();
			}
			emit sig_new_connection(ip);
			break;
	}

	if(close_connection){
		_socket->close();
	}
}
