/* GUI_Library_windowed.cpp */

/* Copyright (C) 2011  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


/*
 * GUI_Library_windowed.cpp
 *
 *  Created on: Apr 24, 2011
 *      Author: luke
 */

#include "GUI_Library_windowed.h"

#include "Helper/DirectoryReader/DirectoryReader.h"
#include "InfoBox/GUILibraryInfoBox.h"
#include "ImportFolderDialog/GUIImportFolder.h"

#include <QMessageBox>
#include <QFileDialog>

GUI_Library_windowed::GUI_Library_windowed(LocalLibrary* library, QWidget* parent) :
	GUI_AbstractLibrary(library, parent),
	Ui::Library_windowed()
{
	setup_parent(this);

	bool show_dir_view = _settings->get(Set::Lib_ShowDirs);

	_lib_info_dialog = new GUI_Library_Info_Box(this);
	_dir_model = nullptr;
	_file_model = nullptr;
	_icon_provider = nullptr;

	_ui_importer = new GUI_ImportFolder(this, true);

	connect(lv_album, SIGNAL(sig_disc_pressed(int)), this, SLOT(disc_pressed(int)));
	connect(lv_album, SIGNAL(sig_import_files(const QStringList&)), this, SLOT(import_files(const QStringList&)));
	connect(lv_artist, SIGNAL(sig_import_files(const QStringList&)), this, SLOT(import_files(const QStringList&)));
	connect(tb_title, SIGNAL(sig_import_files(const QStringList&)), this, SLOT(import_files(const QStringList&)));
	connect(btn_info, SIGNAL(clicked()), _lib_info_dialog, SLOT(psl_refresh()));
	connect(btn_dir_view, SIGNAL(toggled(bool)), this, SLOT(show_dir_view_toggled(bool)));

	connect(_library, SIGNAL(sig_no_library_path()), this, SLOT(lib_no_lib_path()));
	connect(_library, SIGNAL(sig_import_dialog_requested()), this, SLOT(import_dialog_requested()));

	library->load();

	btn_dir_view->setChecked(show_dir_view);
	show_dir_view_toggled(show_dir_view);

	setAcceptDrops(true);
}


GUI_Library_windowed::~GUI_Library_windowed() {

	delete _icon_provider;
	delete _lib_info_dialog;
}


void GUI_Library_windowed::language_changed() {

	retranslateUi(this);

	GUI_AbstractLibrary::language_changed();
}

void GUI_Library_windowed::init_dir_view(){

	QString lib_path = _settings->get(Set::Lib_Path);

	_icon_provider = new IconProvider();
	_dir_model = new QFileSystemModel(this);
	_file_model = new QFileSystemModel(this);

	_dir_model->setRootPath(lib_path);
	_dir_model->setFilter(QDir::NoDotAndDotDot | QDir::AllDirs);
	_dir_model->setIconProvider(_icon_provider);

	_file_model->setRootPath(lib_path);
	_file_model->setFilter(QDir::NoDotAndDotDot | QDir::Files);
	_file_model->setIconProvider(_icon_provider);
	_file_model->setNameFilterDisables(false);
	_file_model->setNameFilters(Helper::get_soundfile_extensions() << Helper::get_playlistfile_extensions());

	lv_directories->setModel(_dir_model);
	lv_directories->setRootIndex(_dir_model->index(lib_path));
	lv_directories->setDragEnabled(true);

	for(int i=1; i<4; i++){
		lv_directories->hideColumn(i);
	}


	lv_files->setModel(_file_model);
	lv_files->setRootIndex(_file_model->index(lib_path));
	lv_files->setDragEnabled(true);
	lv_files->setIconSize(QSize(16, 16));

	connect(lv_directories, SIGNAL(clicked(QModelIndex)), this, SLOT(dir_clicked(QModelIndex)));
	connect(lv_directories, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(dir_clicked(QModelIndex)));
	connect(lv_files, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(file_dbl_clicked(QModelIndex)));
}


LocalLibrary::TrackDeletionMode GUI_Library_windowed::show_delete_dialog(int n_tracks) {

	QMessageBox dialog(this);
	QAbstractButton* clicked_button;
	QPushButton* only_library_button;

	dialog.setFocus();
	dialog.setIcon(QMessageBox::Warning);
	dialog.setText("<b>" + tr("Warning") + "!</b>");
	dialog.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
	only_library_button = dialog.addButton(tr("Only from library"), QMessageBox::AcceptRole);
	dialog.setDefaultButton(QMessageBox::No);
	QString info_text = tr("You are about to delete %1 files").arg(n_tracks);

	dialog.setInformativeText(info_text + "\n" + tr("Continue?") );

	int answer = dialog.exec();
	clicked_button = dialog.clickedButton();
	dialog.close();


	if(answer == QMessageBox::No){
		return LocalLibrary::TrackDeletionMode::None;
	}

	if(answer == QMessageBox::Yes){
		return LocalLibrary::TrackDeletionMode::AlsoFiles;
	}

	if(clicked_button->text() == only_library_button->text()) {
		return LocalLibrary::TrackDeletionMode::OnlyLibrary;
	}

	return LocalLibrary::TrackDeletionMode::None;
}



void GUI_Library_windowed::disc_pressed(int disc) {
	LocalLibrary* ll = (LocalLibrary*) _library;
	ll->psl_disc_pressed(disc);
}


void GUI_Library_windowed::lib_no_lib_path(){

	Message::warning(tr("Please select your library path first and reload again."));

	QString dir = QFileDialog::getExistingDirectory(this, tr("Open Directory"),	QDir::homePath(), QFileDialog::ShowDirsOnly);

	if(dir.size() > 3){
		_settings->set(Set::Lib_Path, dir);
	}
}

void GUI_Library_windowed::import_files(const QStringList& files){
	_library->import_files(files);
}

void GUI_Library_windowed::import_dialog_requested(){
	_ui_importer->show();
}



void GUI_Library_windowed::dir_clicked(QModelIndex idx){

	QString dir = _dir_model->fileInfo(idx).absoluteFilePath();
	QModelIndex tgt_idx = _file_model->setRootPath(dir);
	lv_files->setRootIndex(tgt_idx);
}

void GUI_Library_windowed::dir_dbl_clicked(QModelIndex idx){
	QStringList paths;
	QString path = _dir_model->filePath(idx);
	DirectoryReader reader;
	reader.get_files_in_dir_rec(QDir(path), paths);
	_library->psl_prepare_tracks_for_playlist(paths);
}

void GUI_Library_windowed::file_dbl_clicked(QModelIndex idx){
	QStringList paths;
	paths << _dir_model->filePath(idx);
	_library->psl_prepare_tracks_for_playlist(paths);
}


void GUI_Library_windowed::show_dir_view_toggled(bool b){
	_settings->set(Set::Lib_ShowDirs, b);

	if(!_dir_model){
		init_dir_view();
	}

	lv_files->setVisible(b);
	lv_directories->setVisible(b);

	lv_album->setVisible(!b);
	lv_artist->setVisible(!b);
	tb_title->setVisible(!b);
}

