/* LibraryView.cpp */

/* Copyright (C) 2011  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


/*
 * LibraryView.cpp
 *
 *  Created on: Jun 26, 2011
 *      Author: luke
 */

#include "LibraryView.h"


LibraryView::LibraryView(QWidget* parent) : SearchableTableView(parent) {

	_drag = nullptr;
	_rc_header_menu = nullptr;
	_editor = nullptr;
	_cur_filling = false;

    rc_menu_init();

    connect(this->horizontalHeader(), SIGNAL(sectionClicked(int)), this, SLOT(sort_by_column(int)));
    setAcceptDrops(true);
	setSelectionBehavior(QAbstractItemView::SelectRows);

	clearSelection();
}


LibraryView::~LibraryView() {
    delete _rc_menu;
}


// mouse events
void LibraryView::mousePressEvent(QMouseEvent* event) {

	QPoint pos_org = event->pos();
    QPoint pos = QWidget::mapToGlobal(pos_org);

	if(_model->rowCount() == 0){
		return;
	}

	switch(event->button()) {

		case Qt::LeftButton:

			SearchableTableView::mousePressEvent(event);

			_drag_pos = pos_org;

			break;

		case Qt::RightButton:

			SearchableTableView::mousePressEvent(event);
			pos.setY(pos.y() + 35);
			pos.setX(pos.x() + 10);
			rc_menu_show(pos);

			break;

		case Qt::MidButton:
			SearchableTableView::mousePressEvent(event);

			emit sig_middle_button_clicked(pos);
			break;

		default:
			break;
    }

}

void LibraryView::mouseMoveEvent(QMouseEvent* event) {

	int distance = (event->pos() - _drag_pos).manhattanLength();

	if( event->buttons() & Qt::LeftButton &&
		distance > QApplication::startDragDistance())
	{
		do_drag();
	}
}

void LibraryView::mouseDoubleClickEvent(QMouseEvent *event) {

    event->setModifiers(Qt::NoModifier);
    QTableView::mouseDoubleClickEvent(event);
}

void LibraryView::mouseReleaseEvent(QMouseEvent* event) {

    switch (event->button()) {

        case Qt::LeftButton:

            SearchableTableView::mouseReleaseEvent(event);
            event->accept();

            break;

        default:
            break;
    }
}
// mouse events end


// keyboard events
void LibraryView::keyPressEvent(QKeyEvent* event) {

	int key = event->key();

    Qt::KeyboardModifiers  modifiers = event->modifiers();

	bool shift_pressed = (modifiers & Qt::ShiftModifier);
	bool alt_pressed = (modifiers & Qt::AltModifier);
	bool ctrl_pressed = (modifiers & Qt::ControlModifier);

	if((key == Qt::Key_Up || key == Qt::Key_Down)) {
		if(this->selectionModel()->selection().isEmpty()) {
			if(_model->rowCount() > 0) {
				selectRow(0);
			}
			return;
		}

        if(ctrl_pressed)
            event->setModifiers(Qt::NoModifier);
    }

	SearchableTableView::keyPressEvent(event);
	if(!event->isAccepted()) return;

    QList<int> selections = get_selections();

	switch(key) {

		case Qt::Key_Right:
        case Qt::Key_Plus:
			if(_editor) {
				sp_log(Log::Debug) << "Try to access editor " << _editor->get_id();
                _editor->increase();
            }
			break;

        case Qt::Key_Left:
        case Qt::Key_Minus:
            if(_editor) {
				sp_log(Log::Debug) << "Try to access editor " << _editor->get_id();
                _editor->decrease();
            }
			break;

        case Qt::Key_Escape:

            clearSelection();
            this->selectionModel()->clearSelection();

            break;

        case Qt::Key_Return:
        case Qt::Key_Enter:

            if(selections.size() == 0) break;
            if(ctrl_pressed) break;

            // standard enter
			if(!shift_pressed && !alt_pressed){
				emit doubleClicked( _model->index(selections[0], 0));
			}

            // enter with shift
			else if(shift_pressed && !alt_pressed) {
				emit sig_append_clicked();
            }

			else if(alt_pressed) {
				emit sig_play_next_clicked();
            }

            break;

		case Qt::Key_End:
			this->selectRow(_model->rowCount() - 1);
            break;

        case Qt::Key_Home:
            this->selectRow(0);
            break;

		default:
			break;
    }
}
// keyboard end

void LibraryView::setModel(LibraryItemModel * model) {

	SearchableTableView::setModel( (AbstractSearchTableModel*) model );
    _model = model;
}


// selections
int LibraryView::get_min_selected() {

	QModelIndexList selections = this->selectionModel()->selectedRows();
	auto lambda = [](const QModelIndex& idx1, const QModelIndex& idx2){
		return idx1.row() < idx2.row();
	};
	auto it = std::min_element(selections.begin(), selections.end(), lambda);
	return it->row();
}

void LibraryView::selectionChanged(const QItemSelection& selected, const QItemSelection& deselected ) {

	if(_cur_filling) {
		return;
	}

	SearchableTableView::selectionChanged(selected, deselected);

	QList<int> idx_list_int = get_selections();

	if(selected.indexes().size() > 0) {
        scrollTo(selected.indexes()[0]);
	}

	emit sig_sel_changed(idx_list_int);
}


void LibraryView::save_selections()
{
	QList<int> idx_list_int = get_selections();
	_model->add_selections(idx_list_int);
}
// selections end


template void LibraryView::fill<MetaDataList, MetaData>(const MetaDataList&);
template void LibraryView::fill<ArtistList, Artist>(const ArtistList&);
template void LibraryView::fill<AlbumList, Album>(const AlbumList&);


template < class TList, class T >
void LibraryView::fill(const TList& input_data) {

	QModelIndex idx;

    int size = input_data.size();

	_cur_filling = true;

	_model->remove_all_and_insert(size);
	QList<int> idx_list_int;

	for(int row=0; row < size; row++) {

		QVariant var_data = T::toVariant( input_data[row] );
		if(_model->is_selected(input_data[row].id)){
				idx_list_int << row;
		}

		idx = _model->index(row, 1);
		_model->setData(idx, var_data, Qt::EditRole );
	}

	_model->clear_selections();
	select_rows(idx_list_int, 0, _model->columnCount() - 1);

	_cur_filling = false;
}

void LibraryView::do_drag(){

	CustomMimeData* mimedata = _model->get_mimedata();

	if(_drag){
		delete _drag;
	}

	_drag = new QDrag(this);

	connect(_drag, SIGNAL(destroyed()), this, SLOT(drag_deleted()));

	_drag->setMimeData(mimedata);
	_drag->exec();
}


void LibraryView::drag_deleted() {
	_drag = nullptr;
}

// appearance end

void LibraryView::set_editor(RatingLabel *editor) {

	_editor = editor;
	if(_editor) {
		sp_log(Log::Debug) << "New Editor: " << _editor->get_id();
    }

}

void LibraryView::editorDestroyed(QObject* object) {
	Q_UNUSED(object);
	_editor = nullptr;
}

// drag drop
void LibraryView::dropEvent(QDropEvent *event) {

    event->accept();
    const QMimeData* mime_data = event->mimeData();

    if(!mime_data) return;

	QString text = "";

	if(mime_data->hasText()){
		text = mime_data->text();
	}

    // extern drops
    if( !mime_data->hasUrls() || text.compare("tracks", Qt::CaseInsensitive) == 0) {
        return;
    }

	QStringList filelist;
	for(const QUrl& url : mime_data->urls()) {
        QString path;
        QString url_str = url.toString();
        path =  url_str.right(url_str.length() - 7).trimmed();
        path = path.replace("%20", " ");

		if(QFile::exists(path)) {
            filelist << path;
        }
    } // end foreach

    emit sig_import_files(filelist);

}

void LibraryView::dragEnterEvent(QDragEnterEvent *event) {
    event->accept();
}

void  LibraryView::dragMoveEvent(QDragMoveEvent *event) {
    event->accept();
}
// drag drop end



// Right click stuff
void LibraryView::rc_menu_init() {
    _rc_menu = new ContextMenu(this);
	set_rc_menu(ENTRY_PLAY_NEXT | ENTRY_INFO | ENTRY_DELETE | ENTRY_EDIT | ENTRY_APPEND);
}

void LibraryView::set_rc_menu(int entries){
	if(!_rc_menu) return;
	_rc_menu->setup_entries(entries);
}


void LibraryView::rc_menu_show(const QPoint& p) {

	connect(_rc_menu, SIGNAL(sig_edit_clicked()), this, SIGNAL(sig_edit_clicked()));
	connect(_rc_menu, SIGNAL(sig_info_clicked()), this, SIGNAL(sig_info_clicked()));
	connect(_rc_menu, SIGNAL(sig_delete_clicked()), this, SIGNAL(sig_delete_clicked()));
	connect(_rc_menu, SIGNAL(sig_play_next_clicked()), this, SIGNAL(sig_play_next_clicked()));
	connect(_rc_menu, SIGNAL(sig_append_clicked()), this, SIGNAL(sig_append_clicked()));
	connect(_rc_menu, SIGNAL(sig_refresh_clicked()), this, SIGNAL(sig_refresh_clicked()));

    _rc_menu->exec(p);

	disconnect(_rc_menu, SIGNAL(sig_edit_clicked()), this, SIGNAL(sig_edit_clicked()));
	disconnect(_rc_menu, SIGNAL(sig_info_clicked()), this, SIGNAL(sig_info_clicked()));
	disconnect(_rc_menu, SIGNAL(sig_delete_clicked()), this, SIGNAL(sig_delete_clicked()));
	disconnect(_rc_menu, SIGNAL(sig_play_next_clicked()), this, SIGNAL(sig_play_next_clicked()));
	disconnect(_rc_menu, SIGNAL(sig_append_clicked()), this, SIGNAL(sig_append_clicked()));
	disconnect(_rc_menu, SIGNAL(sig_refresh_clicked()), this, SIGNAL(sig_refresh_clicked()));
}
// right click stuff end
