/* GUI_Simpleplayer.h */

/* Copyright (C) 2011  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef GUI_SIMPLEPLAYER_H
#define GUI_SIMPLEPLAYER_H

#include "GUI/Player/ui_GUI_Player.h"

#include "Components/PlayManager/PlayManager.h"
#include "Components/Engine/GSTEngineHandler.h"
#include "Components/CoverLookup/CoverLookup.h"

#include "GUI/LastFM/GUI_LastFM.h"
#include "GUI/Playlist/GUI_Playlist.h"
#include "GUI/Library/GUI_LibraryPath.h"
#include "GUI/AlternativeCovers/GUI_AlternativeCovers.h"
#include "GUI/Notifications/GUI_Notifications.h"
#include "GUI/StartupDialog/GUI_StartupDialog.h"
#include "GUI/LanguageChooser/GUI_LanguageChooser.h"
#include "GUI/StreamRecorder/GUI_StreamRecorder.h"
#include "GUI/PlayerPlugins/Broadcasting/GUI_BroadcastSetup.h"
#include "GUI/Player/GUI_TrayIcon.h"
#include "GUI/GlobalMessage/GlobalMessage.h"

#include "Interfaces/NotificationInterface/Notification.h"
#include "Interfaces/LibraryInterface/LibraryPluginLoader.h"
#include "Interfaces/PlayerPlugin/PlayerPluginHandler.h"

#include "Helper/WebAccess/AsyncWebAccess.h"

#include <QMainWindow>
#include <QShowEvent>
#include <QCloseEvent>
#include <QTranslator>
#include <QKeySequence>
#include <QAction>


class GUI_Player :
		public SayonaraMainWindow,
		public GlobalMessageReceiver,
		private Ui::Sayonara
{

	Q_OBJECT


public:
    explicit GUI_Player(QTranslator* translator, QWidget *parent=nullptr);
    ~GUI_Player();

	void set_playlist(GUI_Playlist* playlist);
	void set_libraries(LibraryPluginLoader* plugin_loader, int idx);

	void set_info_dialog(GUI_InfoDialog* info_dialog);
	void set_player_plugin_handler(PlayerPluginHandler* pph);

	void ui_loaded();



public slots:

	void set_cur_pos_ms(quint64 pos_ms);
	void id3_tags_changed(const MetaDataList& v_md_old, const MetaDataList& v_md_new);

	void md_changed(const MetaData&);
	void dur_changed(const MetaData&);
	void br_changed(const MetaData&);

	void reload_library_allowed(bool);

	void really_close();


    /**
      * Set current position in filestream
      */

	void tray_item_activated(QSystemTrayIcon::ActivationReason reason);

    /* Plugins */
	void show_plugin(PlayerPlugin* plugin);
	void hide_all_plugins();




private slots:

	void play_clicked();
	void stop_clicked();
	void prev_clicked();
	void next_clicked();
    void rec_clicked(bool);
	void buffering(int progress);

	void played();
	void paused();
	void stopped();
	void playstate_changed(PlayManager::PlayState);

	void track_changed(const MetaData& md);

	void cover_clicked();
	void seek(int);
    void jump_forward();
    void jump_backward();
    void jump_forward_ms();
    void jump_backward_ms();

	void mute_button_clicked();
	void volume_changed(int volume_percent);
	void change_volume_by_tick(int val);
	void increase_volume();
	void decrease_volume();

    /* File */
	void open_files_clicked();
	void open_dir_clicked();
	void reload_library_clicked();
	void import_dir_clicked();
	void import_files_clicked();


    /* View */
	void show_library(bool, bool resize=true);
    void small_playlist_items_toggled(bool);
    void show_notification_toggled(bool);
    void show_fullscreen_toggled(bool);
	void _sl_fullscreen_toggled();
	void skin_toggled(bool);


    /* Preferences */
	void show_language_chooser();

	void set_library_path_clicked();
	void setup_lastfm_clicked();
	void setup_streamrecorder_clicked();
	void setup_broadcasting_clicked();

    void load_pl_on_startup_toggled(bool);
    void min2tray_toggled(bool);
    void only_one_instance_toggled(bool);
	void only_show_tracks_toggled(bool);
	void live_search_toggled(bool);
	void notify_new_version_toggled(bool);

	void about();
	void help();

	void set_standard_cover();
	void set_cover_image(const CoverLocation&);
	void fetch_cover();

	void awa_version_finished(bool success);
	void awa_translators_finished(bool success);
	void set_cur_library(int idx);

	void _sl_libpath_changed();
	void _sl_sr_active_changed();


protected:
	GUI_Playlist*				ui_playlist;

	QList<GUI_AbstractLibrary*> ui_libraries;
	GUI_AbstractLibrary*		_cur_library;
	AbstractLibrary*			_local_library;

	GUI_LibraryPath*			ui_libpath;
	GUI_Notifications*			ui_notifications;
	GUI_StartupDialog*			ui_StartupDialog;
	GUI_LanguageChooser*		ui_language_chooser;
	GUI_StreamRecorder*			ui_streamrecorder;
	GUI_BroadcastSetup*			ui_broadcasting;
	GUI_LastFM*					ui_lastfm;
	GUI_AlternativeCovers*		_ui_alternative_covers;

	GSTEngineHandler*			_engine;
	CoverLookup*				_cov_lookup;
	PlayerPluginHandler*		_pph;


	AsyncWebAccess*				_awa_version;
	AsyncWebAccess*				_awa_translators;

	bool						_mute;
	GUI_TrayIcon *				_tray_icon;

	QString						_skin_suffix;

	MetaData					_md;
	MetaData					_md_corrected;
	bool						_md_available;

	int							_ui_library_width;
	int							_library_stretch_factor;


	QTranslator*				_translator;
	QStringList					_translators;

	PlayManager*				_play_manager;

	void init_gui();

	void setup_tray_actions ();
	void setup_volume_button(int percent);
	void setup_connections();

	void set_album_label();
	void set_radio_mode(RadioMode model);


	bool check_library_path();

	QAction* create_action(QKeySequence key_sequence);
	QAction* create_actions(QList<QKeySequence>& key_sequences);

	void showEvent(QShowEvent *e);
	void closeEvent(QCloseEvent* e);
	void keyPressEvent(QKeyEvent* e);
	void resizeEvent(QResizeEvent* e);
	void moveEvent(QMoveEvent* e);


	virtual void language_changed();

	virtual void skin_changed();

	void set_total_time_label(qint64 length_ms);
	virtual void set_cur_pos_label(int val);


	// Methods for other mudules to display info/warning/error
	virtual GlobalMessage::Answer error_received(const QString &error, const QString &sender_name=QString());
	virtual GlobalMessage::Answer warning_received(const QString &error, const QString &sender_name=QString());
	virtual GlobalMessage::Answer info_received(const QString &error, const QString &sender_name=QString());
	virtual GlobalMessage::Answer question_received(const QString &info, const QString &sender_name=QString(), GlobalMessage::QuestionType type=GlobalMessage::QuestionType::YesNo);
};


#endif // GUI_SIMPLEPLAYER_H
