/* PlaylistView.cpp */

/* Copyright (C) 2011  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


/*
 * PlaylistView.cpp
 *
 *  Created on: Jun 26, 2011
 *      Author: luke
 */

#include "PlaylistView.h"

#include "Helper/CustomMimeData.h"
#include "Helper/DirectoryReader/DirectoryReader.h"
#include "Helper/Helper.h"

#include "GUI/ContextMenu/ContextMenu.h"
#include "GUI/Playlist/delegate/PlaylistItemDelegate.h"

#include <QScrollBar>
#include <QPainter>

PlaylistView::PlaylistView(QWidget* parent) :
	SearchableListView(parent),
	Ui::GUI_PlaylistView()
{

	setupUi(this);

	_model = new PlaylistItemModel(this);
	_delegate = new PlaylistItemDelegate(this, true);

	_drag = nullptr;
    _inner_drag_drop = false;
	_saved_selection_before_removal = -1;

	init_rc_menu();

    this->setModel(_model);
    this->setAbstractModel(_model);
	this->setItemDelegate(_delegate);
    this->setDragEnabled(true);
	this->setDropIndicatorShown(true);
	this->setDragDropMode(QAbstractItemView::DragDrop);
    this->setAcceptDrops(true);

    this->setSelectionRectVisible(true);
    this->setAlternatingRowColors(true);
    this->setMovement(QListView::Free);

    connect(this, SIGNAL(pressed(const QModelIndex&)), this, SLOT(row_pressed(const QModelIndex&)));
    connect(this, SIGNAL(doubleClicked(const QModelIndex&)), this, SLOT(row_double_clicked(const QModelIndex&)));
    connect(this, SIGNAL(clicked(const QModelIndex&)), this, SLOT(row_released(const QModelIndex&)));
}

PlaylistView::~PlaylistView() {

	if(_rc_menu){
		delete _rc_menu;
	}

    delete _model;
}


void PlaylistView::mousePressEvent(QMouseEvent* event) {

    QPoint pos_org = event->pos();
    QPoint pos = QWidget::mapToGlobal(pos_org);
    QItemSelection sel, desel;

	if(_model->rowCount() == 0){
		return;
	}

    switch (event->button()) {

		case Qt::LeftButton:
			_sel_changed = false;
			SearchableListView::mousePressEvent(event);
			if(!_sel_changed) {
				selectionChanged(sel, desel);
			}

			_drag_pos = event->pos();

			break;

		case Qt::RightButton:
			_sel_changed = false;

			SearchableListView::mousePressEvent(event);

			if(!_sel_changed) {
				selectionChanged(sel, desel);
			}

			pos.setY(pos.y());
			pos.setX(pos.x() + 10);

			_rc_menu->exec(pos);

			break;

		default:
			break;
    }
}


void PlaylistView::mouseMoveEvent(QMouseEvent* event) {

	int distance = (event->pos() - _drag_pos).manhattanLength();

	if( event->buttons() & Qt::LeftButton &&
		distance >= QApplication::startDragDistance())
	{
		CustomMimeData* mimedata = _model->get_custom_mimedata(this->selectedIndexes());

		if(_drag){
			delete _drag;
		}

		_drag = new QDrag(this);

		connect(_drag, SIGNAL(destroyed()), this, SLOT(drag_deleted()));

		_drag->setMimeData(mimedata);
		_drag->exec(Qt::CopyAction);
	}
}

void PlaylistView::drag_deleted(){
	_drag = nullptr;
}


void PlaylistView::mouseReleaseEvent(QMouseEvent* event) {

	switch (event->button()) {

		case Qt::LeftButton:

			SearchableListView::mouseReleaseEvent(event);
			event->accept();

			break;

		default:
			break;
	}
}


// get the min index of selected rows
int PlaylistView::get_min_selected() {

    QModelIndexList lst = this->selectionModel()->selectedRows();

    if(lst.size() == 0) {
        return 0;
    }

	auto lambda_compare = [](const QModelIndex& idx1, const QModelIndex& idx2)
	{
		return idx1.row() < idx2.row();
	};

	auto it = std::min_element( lst.begin(), lst.end(), lambda_compare );

	return it->row();

}

// mark row as currently pressed
void PlaylistView::goto_row(int row) {

	if( (row >= _model->rowCount()) || (row < 0) ) {
		return;
	}

    QModelIndex idx = _model->index(row, 0);
    row_released(idx);

	this->scrollTo(idx, SearchableListView::EnsureVisible);
}


void PlaylistView::keyPressEvent(QKeyEvent* event) {

    int key = event->key();

	if((key == Qt::Key_Up || key == Qt::Key_Down)) {
		if(this->selectionModel()->selection().isEmpty()) {
			if(_model->rowCount() > 0) select_row(0);
			return;
		}
	}

    Qt::KeyboardModifiers  modifiers = event->modifiers();
	SearchableListView::keyPressEvent(event);

	if(!event->isAccepted() ) return;

    int new_row = -1;
    int min_row = get_min_selected();

	switch(key) {
		case Qt::Key_A:
			if( modifiers & Qt::ControlModifier ) {
				select_all();
			}
			break;

		case Qt::Key_Delete:
			remove_cur_selected_rows();
			break;

		case Qt::Key_End:
			new_row = _model->rowCount() - 1;
			break;

		case Qt::Key_Home:
			new_row = 0;
			break;

		case Qt::Key_Return:
		case Qt::Key_Enter:
			this->sig_double_clicked(min_row);
			break;

		default:
			break;
    }

	if(new_row != -1) {
		goto_row(new_row);
	}
}

void PlaylistView::resizeEvent(QResizeEvent *e) {

    this->set_delegate_max_width(_model->rowCount());
    e->accept();

}

void PlaylistView::init_rc_menu() {

    _rc_menu = new ContextMenu(this);

	connect(_rc_menu, SIGNAL(sig_info_clicked()), this, SIGNAL(sig_info_clicked()));
	connect(_rc_menu, SIGNAL(sig_edit_clicked()), this, SIGNAL(sig_edit_clicked()));
	connect(_rc_menu, SIGNAL(sig_remove_clicked()), this, SIGNAL(sig_remove_clicked()));
}

void PlaylistView::set_context_menu_actions(int actions) {
    _rc_menu->setup_entries(actions);
}

int PlaylistView::get_num_rows() {
    return _model->rowCount();
}

void PlaylistView::set_current_track(int row) {

	for(int i=0; i<_model->rowCount(); i++) {

		QModelIndex idx = _model->index(i);
        MetaData md;
        QVariant v = _model->data(idx, Qt::WhatsThisRole);

		if( !MetaData::fromVariant(v, md) ) {
			continue;
		}

        md.pl_playing = (row == i);

	    _model->setData(idx, MetaData::toVariant(md), Qt::EditRole);
    }

    QModelIndex new_idx = _model->index(row);
	scrollTo(new_idx, SearchableListView::EnsureVisible);
}


void PlaylistView::clear() {
    clear_selection();
    _model->removeRows(0, _model->rowCount());
}

#define IGNORE_SELECTION_CHANGES(b) _ignore_selection_changes = b

void PlaylistView::fill(const MetaDataList &v_md) {

    this->set_delegate_max_width(v_md.size());
    _cur_selected_rows.clear();

    IGNORE_SELECTION_CHANGES(true);
        _model->removeRows(0, _model->rowCount());
    IGNORE_SELECTION_CHANGES(false);

    if(v_md.size() == 0) return;

    IGNORE_SELECTION_CHANGES(true);
        _model->insertRows(0, v_md.size());
    IGNORE_SELECTION_CHANGES(false);

    QList<int> selected_rows;

    QModelIndex idx_cur_playing = _model->index(0);
	if(_saved_selection_before_removal >= 0){
		selected_rows << _saved_selection_before_removal;
	}

    for(int i=0; i<v_md.size(); i++) {
		
		const MetaData& md = v_md[i];
        QModelIndex model_idx = _model->index(i, 0);
        
		if(md.pl_playing) {
			idx_cur_playing = model_idx;
		}

		if(md.pl_selected) {
            selected_rows << i;
		}

	    _model->setData(model_idx, MetaData::toVariant(md), Qt::EditRole);
    }

	_saved_selection_before_removal = -1;

	this->select_rows(selected_rows, 0, 0);
	this->scrollTo(idx_cur_playing, SearchableListView::EnsureVisible);
}

void PlaylistView::row_pressed(const QModelIndex& idx) {
	Q_UNUSED(idx)
	_inner_drag_drop = true;
}

void PlaylistView::row_released(const QModelIndex& idx) {
	Q_UNUSED(idx)
    _inner_drag_drop = false;
}

void PlaylistView::row_double_clicked(const QModelIndex& idx) {
    _inner_drag_drop = false;

    if(idx.isValid()) emit sig_double_clicked(idx.row());
}

void PlaylistView::clear_selection() {

    this->selectionModel()->clearSelection();
    this->clearSelection();
}

void PlaylistView::select_row(int i) {

	if(_model->rowCount() == 0){
		return;
	}

	i = std::min(_model->rowCount(), i);
	i = std::max(i, 0);

	this->selectionModel()->setCurrentIndex(_model->index(i), QItemSelectionModel::Select);

    QList<int> lst;
    lst << i;
	select_rows(lst, 0, 0);
}

void PlaylistView::select_rows(const QList<int>& lst, int min_col, int max_col) {
	SayonaraSelectionView::select_rows(lst, min_col, max_col);
    _cur_selected_rows = lst;
}


void PlaylistView::select_all() {
    selectAll();
}

void PlaylistView::selectionChanged ( const QItemSelection & selected, const QItemSelection & deselected ) {

    if(_ignore_selection_changes) return;

	SearchableListView::selectionChanged(selected, deselected);

    MetaDataList v_md;
	_cur_selected_rows = get_selections();

	_model->set_selected(_cur_selected_rows);

	if(selected.indexes().size() > 0){
		scrollTo(selected.indexes()[0], SearchableListView::EnsureVisible);
    }

	set_context_menu_actions(ENTRY_INFO | ENTRY_REMOVE);

    _sel_changed = true;

	_model->get_metadata(_cur_selected_rows, v_md);
	emit sig_sel_changed(v_md, _cur_selected_rows);
}


// remove the black line under the titles
void  PlaylistView::clear_drag_lines(int row) {

	_delegate->set_drag_index(-1);
	this->update(_model->index(row));
}


int PlaylistView::calc_dd_line(QPoint pos) {

    if(pos.y() < 0) {
        return -1;
    }

    int row = this->indexAt(pos).row();

	if(row <= -1) {
		row = _model->rowCount()-1;
	}

    return row;
}


// the drag comes, if there's data --> accept it
void PlaylistView::dragEnterEvent(QDragEnterEvent* event) {
    event->accept();
}

void PlaylistView::dragMoveEvent(QDragMoveEvent* event) {

	event->accept();

	int row = calc_dd_line(event->pos() );
	bool is_old = _delegate->is_drag_index(row);

	if(!is_old){
		clear_drag_lines(_delegate->get_drag_index());
		_delegate->set_drag_index(row);
		this->update(_model->index(row));
	}
}


// we start the drag action, all lines has to be cleared
void PlaylistView::dragLeaveEvent(QDragLeaveEvent* event) {

	event->accept();
	clear_drag_lines(_delegate->get_drag_index());
}

// called from GUI_Playlist when data has not been dropped
// directly into the view widget. Insert on first row then
void PlaylistView::dropEventFromOutside(QDropEvent* event) {

	event->accept();

	if(!event->mimeData()) {
		return;
	}

	handle_drop(event, true);
}


void PlaylistView::dropEvent(QDropEvent* event) {

	event->accept();

	if(!event->mimeData()) {
		return;
	}

	handle_drop(event, false);
}


void PlaylistView::handle_drop(QDropEvent* event, bool from_outside) {

    QList<int> affected_rows;
	MetaDataList v_md;
	QString text;

	// where did i drop?
	int row = _delegate->get_drag_index();
	clear_drag_lines(row);

	if(from_outside) {
		row = -1;
    }

	if(_inner_drag_drop) {
        _inner_drag_drop = false;

		if( _cur_selected_rows.contains(row) ) {
            event->ignore();
            return;
        }

		emit sig_rows_moved(_cur_selected_rows, row + 1);
        return;
    }

	const QMimeData* mimedata = event->mimeData();
	const CustomMimeData* custom_mimedata = dynamic_cast<const CustomMimeData*>(mimedata);
	if(!mimedata) {
		return;
	}

	if(mimedata->hasText()) {
		text = mimedata->text();
	}

    // extern
	if( mimedata->hasUrls() && text.compare("tracks", Qt::CaseInsensitive) != 0) {
        QStringList filelist;

		for(const QUrl& url : mimedata->urls()) {
            QString path;
            QString url_str = url.toString();
            path =  url_str.right(url_str.length() - 7).trimmed();
            path = path.replace("%20", " ");
			filelist << std::move(path);

        } // end foreach

        DirectoryReader reader;
		reader.set_filter(Helper::get_soundfile_extensions());

		v_md = reader.get_md_from_filelist(filelist);

		if(v_md.isEmpty()) {
			return;
		}
    }

	else if(mimedata->hasHtml()) {

	}
	else if(mimedata->hasImage()) {

	}

	else if(custom_mimedata != nullptr){
		if(custom_mimedata->hasText() && custom_mimedata->hasMetaData()) {
			size_t sz = custom_mimedata->getMetaData(v_md);
			if(sz == 0) {
				return;
			}
		}
    }

	for(int i=0; i<v_md.size(); i++) {
        affected_rows << i + row + 1;
    }

	emit sig_metadata_dropped(v_md, row + 1);
}


void PlaylistView::scrollUp() {
    QPoint p(5, 5);
    int cur_row = this->indexAt(p).row();
    if(cur_row <= 0) return;

	this->scrollTo(_model->index(cur_row - 1), SearchableListView::EnsureVisible);
}

void PlaylistView::scrollDown() {
    QPoint p(5, this->y() + this->height() - 5);
    int cur_row = this->indexAt(p).row();
    if(cur_row <= 0) return;

	this->scrollTo(_model->index(cur_row - 1), SearchableListView::EnsureVisible);
}

void PlaylistView::remove_cur_selected_rows() {

	_saved_selection_before_removal = -1;

	int row_count = _model->rowCount();

	auto it = std::min_element(_cur_selected_rows.begin(), _cur_selected_rows.end());
	if(it != _cur_selected_rows.end()){
		_saved_selection_before_removal = *it;
	}

	if(_saved_selection_before_removal == row_count - 1 &&
		row_count >= 1)
	{
		_saved_selection_before_removal -= 1;
	}

	emit sig_rows_removed(_cur_selected_rows);
}


void PlaylistView::show_big_items(bool big) {

    if(_delegate) delete _delegate;
    _delegate = new PlaylistItemDelegate(this, !big);

    this->set_delegate_max_width(_model->rowCount());
    this->setItemDelegate(_delegate);

    this->reset();
}


void PlaylistView::set_delegate_max_width(int n_items) {

    bool scrollbar_visible = (( n_items * _delegate->rowHeight() ) >= this->height());

    int max_width = this->width();
	if(scrollbar_visible){
		max_width -= verticalScrollBar()->width();
	}

    _delegate->setMaxWidth(max_width);
}


void PlaylistView::set_playlist_type(PlaylistType type){
	_playlist_type = type;
}

PlaylistType PlaylistView::get_playlist_type(){
	return _playlist_type;
}
