/* id3.cpp */

/* Copyright (C) 2011  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "Helper/Tagging/ID3_Fileheader.h"
#include "Helper/Tagging/id3.h"
#include "Helper/Tagging/id3access.h"
#include "Helper/Helper.h"
#include "DatabaseAccess/CDatabaseConnector.h"

#include <QDir>
#include <QFile>

#include <taglib/tag.h>
#include <taglib/taglib.h>
#include <taglib/fileref.h>


using namespace Helper;

bool ID3::getMetaDataOfFile(MetaData& md) {

	QFile qf(md.filepath());
    md.filesize = qf.size();
    qf.close();

	TagLib::FileRef f(TagLib::FileName(md.filepath().toUtf8()), true, TagLib::AudioProperties::Accurate);

	if(f.isNull() || !f.tag() || !f.file()->isValid() || !f.file()->isReadable(md.filepath().toUtf8()) ) {
		sp_log(Log::Info) << md.filepath() << ": Something's wrong with this file";
		return false;
	}

	TagLib::Tag* tag = f.tag();

	QString artist = QString::fromUtf8(tag->artist().toCString(true));
	QString album = QString::fromUtf8(tag->album().toCString(true));
	QString title = QString::fromUtf8(tag->title().toCString(true));
	QString genre = QString::fromUtf8(tag->genre().toCString(true));
	QString comment = QString::fromUtf8(tag->comment().toCString(true));

    int discnumber = 0;
    int n_discs = 0;

    taglib_id3_extract_discnumber(f, &discnumber);

	uint year = tag->year();
	uint track = tag->track();

	int bitrate = f.audioProperties()->bitrate() * 1000;
	int length = f.audioProperties()->length() * 1000;


    QStringList genres;
	QString genre_str = cvtQString2FirstUpper(genre);
    genres = genre_str.split(QRegExp(",|/|;|\\."));
    for(int i=0; i<genres.size(); i++) {
        genres[i] = genres[i].trimmed();
    }

	md.album = album;
	md.artist = artist;
	md.title = title;
	md.length_ms = length;
	md.year = year;
	md.track_num = track;
	md.bitrate = bitrate;
    md.genres = genres;
    md.discnumber = discnumber;
    md.n_discs = n_discs;
	md.comment = comment;

	if(md.title.length() == 0) {
		QString dir, filename;
		Helper::split_filename(md.filepath(), dir, filename);

		if(filename.size() > 4){
			filename = filename.left(filename.length() - 4);
		}

		md.title = filename;
	}

	return true;
}


bool ID3::setMetaDataOfFile(MetaData& md) {

	QString filepath = md.filepath();
	TagLib::FileRef f(TagLib::FileName(filepath.toUtf8()));
	if(f.isNull() || !f.tag() || !f.file()->isValid() || !f.file()->isWritable(filepath.toUtf8()) ) {
        sp_log(Log::Info) << "ID3 cannot save";
		return false;
	}

	TagLib::String album(md.album.toUtf8().data(), TagLib::String::UTF8);
	TagLib::String artist(md.artist.toUtf8().data(), TagLib::String::UTF8);
	TagLib::String title(md.title.toUtf8().data(), TagLib::String::UTF8);
    TagLib::String genre(md.genres.join(",").toUtf8().data(), TagLib::String::UTF8);

	f.tag()->setAlbum(album);
	f.tag()->setArtist(artist);
	f.tag()->setTitle(title);
	f.tag()->setGenre(genre);
	f.tag()->setYear(md.year);
	f.tag()->setTrack(md.track_num);
    f.save();

    TagLib::File* file = f.file();
    TagLib::MPEG::File* f_mp3;

    f_mp3 = dynamic_cast<TagLib::MPEG::File*>(file);

    if(!f_mp3) {
        sp_log(Log::Info) << "Tagging: no mp3 header";
		return true;
    }

    TagLib::ID3v2::Tag* id3_tag = f_mp3->ID3v2Tag();
    if(!id3_tag) {
        sp_log(Log::Info) << "Tagging: no valid id3 tag";
		return true;
    }

	ID3_FileHeader fh(md.filepath());

	if(fh.is_valid()){
		id3_write_discnumber(fh, md.discnumber, md.n_discs);
	}

	return true;
}
